// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.WindowsAzure.Management.TrafficManager;
using Microsoft.WindowsAzure.Management.TrafficManager.Models;

namespace Microsoft.WindowsAzure.Management.TrafficManager
{
    /// <summary>
    /// The Windows Azure Traffic Manager management API provides a RESTful set
    /// of web services that interact with Windows Azure Traffic Manager
    /// service for creating, updating, listing, and deleting Traffic Manager
    /// profiles and definitions.  (see
    /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166981.aspx for
    /// more information)
    /// </summary>
    public static partial class DefinitionOperationsExtensions
    {
        /// <summary>
        /// Creates a new definition for a specified profile.  (see
        /// http://msdn.microsoft.com/en-us/library/hh758257.aspx for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.TrafficManager.IDefinitionOperations.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the profile to create a new definition for.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Definition operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Create(this IDefinitionOperations operations, string profileName, DefinitionCreateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDefinitionOperations)s).CreateAsync(profileName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Creates a new definition for a specified profile.  (see
        /// http://msdn.microsoft.com/en-us/library/hh758257.aspx for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.TrafficManager.IDefinitionOperations.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the profile to create a new definition for.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Definition operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> CreateAsync(this IDefinitionOperations operations, string profileName, DefinitionCreateParameters parameters)
        {
            return operations.CreateAsync(profileName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns an existing profile definition.  (see
        /// http://msdn.microsoft.com/en-us/library/hh758248.aspx for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.TrafficManager.IDefinitionOperations.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the profile to get definition from.
        /// </param>
        /// <returns>
        /// The Get Definition operation response.
        /// </returns>
        public static DefinitionGetResponse Get(this IDefinitionOperations operations, string profileName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDefinitionOperations)s).GetAsync(profileName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns an existing profile definition.  (see
        /// http://msdn.microsoft.com/en-us/library/hh758248.aspx for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.TrafficManager.IDefinitionOperations.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the profile to get definition from.
        /// </param>
        /// <returns>
        /// The Get Definition operation response.
        /// </returns>
        public static Task<DefinitionGetResponse> GetAsync(this IDefinitionOperations operations, string profileName)
        {
            return operations.GetAsync(profileName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns all definitions of a profile  (see
        /// http://msdn.microsoft.com/en-us/library/hh758252.aspx for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.TrafficManager.IDefinitionOperations.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the profile to return all definitions
        /// </param>
        /// <returns>
        /// The List Definitions operation response.
        /// </returns>
        public static DefinitionsListResponse List(this IDefinitionOperations operations, string profileName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDefinitionOperations)s).ListAsync(profileName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns all definitions of a profile  (see
        /// http://msdn.microsoft.com/en-us/library/hh758252.aspx for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.TrafficManager.IDefinitionOperations.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the profile to return all definitions
        /// </param>
        /// <returns>
        /// The List Definitions operation response.
        /// </returns>
        public static Task<DefinitionsListResponse> ListAsync(this IDefinitionOperations operations, string profileName)
        {
            return operations.ListAsync(profileName, CancellationToken.None);
        }
    }
}
