// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Net.Http;
using Hyak.Common;
using Hyak.Common.Internals;
using Microsoft.WindowsAzure.WebSitesExtensions;
using Microsoft.WindowsAzure.WebSitesExtensions.Models;

namespace Microsoft.WindowsAzure.WebSitesExtensions
{
    /// <summary>
    /// The websites extensions client manages the web sites deployments, web
    /// jobs and other extensions.
    /// </summary>
    public partial class WebSiteExtensionsClient : ServiceClient<WebSiteExtensionsClient>, IWebSiteExtensionsClient
    {
        private string _apiVersion;
        
        /// <summary>
        /// Gets the API version.
        /// </summary>
        public string ApiVersion
        {
            get { return this._apiVersion; }
        }
        
        private Uri _baseUri;
        
        /// <summary>
        /// Gets the URI used as the base for all cloud service requests.
        /// </summary>
        public Uri BaseUri
        {
            get { return this._baseUri; }
        }
        
        private BasicAuthenticationCloudCredentials _credentials;
        
        /// <summary>
        /// TBD.
        /// </summary>
        public BasicAuthenticationCloudCredentials Credentials
        {
            get { return this._credentials; }
            set { this._credentials = value; }
        }
        
        private int _longRunningOperationInitialTimeout;
        
        /// <summary>
        /// Gets or sets the initial timeout for Long Running Operations.
        /// </summary>
        public int LongRunningOperationInitialTimeout
        {
            get { return this._longRunningOperationInitialTimeout; }
            set { this._longRunningOperationInitialTimeout = value; }
        }
        
        private int _longRunningOperationRetryTimeout;
        
        /// <summary>
        /// Gets or sets the retry timeout for Long Running Operations.
        /// </summary>
        public int LongRunningOperationRetryTimeout
        {
            get { return this._longRunningOperationRetryTimeout; }
            set { this._longRunningOperationRetryTimeout = value; }
        }
        
        private string _siteName;
        
        /// <summary>
        /// The site name.
        /// </summary>
        public string SiteName
        {
            get { return this._siteName; }
            set { this._siteName = value; }
        }
        
        private ICommandsOperations _commands;
        
        /// <summary>
        /// Operations for invoking commands.
        /// </summary>
        public virtual ICommandsOperations Commands
        {
            get { return this._commands; }
        }
        
        private IContinuousWebJobOperations _continuousWebJobs;
        
        /// <summary>
        /// Operations for managing continuous WebJobs.
        /// </summary>
        public virtual IContinuousWebJobOperations ContinuousWebJobs
        {
            get { return this._continuousWebJobs; }
        }
        
        private IDeploymentOperations _deployments;
        
        /// <summary>
        /// Operations for managing the repositories.
        /// </summary>
        public virtual IDeploymentOperations Deployments
        {
            get { return this._deployments; }
        }
        
        private IDiagnosticOperations _diagnostics;
        
        /// <summary>
        /// Operations for managing the diagnostics settings.
        /// </summary>
        public virtual IDiagnosticOperations Diagnostics
        {
            get { return this._diagnostics; }
        }
        
        private IRepositoryOperations _repository;
        
        /// <summary>
        /// Operations for managing the repository.
        /// </summary>
        public virtual IRepositoryOperations Repository
        {
            get { return this._repository; }
        }
        
        private ISettingsOperations _settings;
        
        /// <summary>
        /// Operations for managing the settings.
        /// </summary>
        public virtual ISettingsOperations Settings
        {
            get { return this._settings; }
        }
        
        private ITriggeredWebJobOperations _triggeredWebJobs;
        
        /// <summary>
        /// Operations for managing Triggered WebJobs.
        /// </summary>
        public virtual ITriggeredWebJobOperations TriggeredWebJobs
        {
            get { return this._triggeredWebJobs; }
        }
        
        /// <summary>
        /// Initializes a new instance of the WebSiteExtensionsClient class.
        /// </summary>
        public WebSiteExtensionsClient()
            : base()
        {
            this._commands = new CommandsOperations(this);
            this._continuousWebJobs = new ContinuousWebJobOperations(this);
            this._deployments = new DeploymentOperations(this);
            this._diagnostics = new DiagnosticOperations(this);
            this._repository = new RepositoryOperations(this);
            this._settings = new SettingsOperations(this);
            this._triggeredWebJobs = new TriggeredWebJobOperations(this);
            this._apiVersion = "2";
            this._longRunningOperationInitialTimeout = -1;
            this._longRunningOperationRetryTimeout = -1;
            this.HttpClient.Timeout = TimeSpan.FromSeconds(300);
        }
        
        /// <summary>
        /// Initializes a new instance of the WebSiteExtensionsClient class.
        /// </summary>
        /// <param name='siteName'>
        /// Required. The site name.
        /// </param>
        /// <param name='credentials'>
        /// Required. TBD.
        /// </param>
        /// <param name='baseUri'>
        /// Optional. Gets the URI used as the base for all cloud service
        /// requests.
        /// </param>
        public WebSiteExtensionsClient(string siteName, BasicAuthenticationCloudCredentials credentials, Uri baseUri)
            : this()
        {
            if (siteName == null)
            {
                throw new ArgumentNullException("siteName");
            }
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            if (baseUri == null)
            {
                throw new ArgumentNullException("baseUri");
            }
            this._siteName = siteName;
            this._credentials = credentials;
            this._baseUri = baseUri;
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Initializes a new instance of the WebSiteExtensionsClient class.
        /// </summary>
        /// <param name='siteName'>
        /// Required. The site name.
        /// </param>
        /// <param name='credentials'>
        /// Required. TBD.
        /// </param>
        public WebSiteExtensionsClient(string siteName, BasicAuthenticationCloudCredentials credentials)
            : this()
        {
            if (siteName == null)
            {
                throw new ArgumentNullException("siteName");
            }
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            this._siteName = siteName;
            this._credentials = credentials;
            this._baseUri = TypeConversion.TryParseUri("https://" + this.SiteName + ".scm.azurewebsites.net:443");
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Initializes a new instance of the WebSiteExtensionsClient class.
        /// </summary>
        /// <param name='httpClient'>
        /// The Http client
        /// </param>
        public WebSiteExtensionsClient(HttpClient httpClient)
            : base(httpClient)
        {
            this._commands = new CommandsOperations(this);
            this._continuousWebJobs = new ContinuousWebJobOperations(this);
            this._deployments = new DeploymentOperations(this);
            this._diagnostics = new DiagnosticOperations(this);
            this._repository = new RepositoryOperations(this);
            this._settings = new SettingsOperations(this);
            this._triggeredWebJobs = new TriggeredWebJobOperations(this);
            this._apiVersion = "2";
            this._longRunningOperationInitialTimeout = -1;
            this._longRunningOperationRetryTimeout = -1;
            this.HttpClient.Timeout = TimeSpan.FromSeconds(300);
        }
        
        /// <summary>
        /// Initializes a new instance of the WebSiteExtensionsClient class.
        /// </summary>
        /// <param name='siteName'>
        /// Required. The site name.
        /// </param>
        /// <param name='credentials'>
        /// Required. TBD.
        /// </param>
        /// <param name='baseUri'>
        /// Optional. Gets the URI used as the base for all cloud service
        /// requests.
        /// </param>
        /// <param name='httpClient'>
        /// The Http client
        /// </param>
        public WebSiteExtensionsClient(string siteName, BasicAuthenticationCloudCredentials credentials, Uri baseUri, HttpClient httpClient)
            : this(httpClient)
        {
            if (siteName == null)
            {
                throw new ArgumentNullException("siteName");
            }
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            if (baseUri == null)
            {
                throw new ArgumentNullException("baseUri");
            }
            this._siteName = siteName;
            this._credentials = credentials;
            this._baseUri = baseUri;
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Initializes a new instance of the WebSiteExtensionsClient class.
        /// </summary>
        /// <param name='siteName'>
        /// Required. The site name.
        /// </param>
        /// <param name='credentials'>
        /// Required. TBD.
        /// </param>
        /// <param name='httpClient'>
        /// The Http client
        /// </param>
        public WebSiteExtensionsClient(string siteName, BasicAuthenticationCloudCredentials credentials, HttpClient httpClient)
            : this(httpClient)
        {
            if (siteName == null)
            {
                throw new ArgumentNullException("siteName");
            }
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            this._siteName = siteName;
            this._credentials = credentials;
            this._baseUri = TypeConversion.TryParseUri("https://" + this.SiteName + ".scm.azurewebsites.net:443");
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Clones properties from current instance to another
        /// WebSiteExtensionsClient instance
        /// </summary>
        /// <param name='client'>
        /// Instance of WebSiteExtensionsClient to clone to
        /// </param>
        protected override void Clone(ServiceClient<WebSiteExtensionsClient> client)
        {
            base.Clone(client);
            
            if (client is WebSiteExtensionsClient)
            {
                WebSiteExtensionsClient clonedClient = ((WebSiteExtensionsClient)client);
                
                clonedClient._siteName = this._siteName;
                clonedClient._credentials = this._credentials;
                clonedClient._baseUri = this._baseUri;
                clonedClient._apiVersion = this._apiVersion;
                clonedClient._longRunningOperationInitialTimeout = this._longRunningOperationInitialTimeout;
                clonedClient._longRunningOperationRetryTimeout = this._longRunningOperationRetryTimeout;
                
                clonedClient.Credentials.InitializeServiceClient(clonedClient);
            }
        }
        
        /// <summary>
        /// Parse enum values for type WebJobType.
        /// </summary>
        /// <param name='value'>
        /// The value to parse.
        /// </param>
        /// <returns>
        /// The enum value.
        /// </returns>
        internal static WebJobType ParseWebJobType(string value)
        {
            if ("triggered".Equals(value, StringComparison.OrdinalIgnoreCase))
            {
                return WebJobType.Triggered;
            }
            if ("continuous".Equals(value, StringComparison.OrdinalIgnoreCase))
            {
                return WebJobType.Continuous;
            }
            throw new ArgumentOutOfRangeException("value");
        }
        
        /// <summary>
        /// Convert an enum of type WebJobType to a string.
        /// </summary>
        /// <param name='value'>
        /// The value to convert to a string.
        /// </param>
        /// <returns>
        /// The enum value as a string.
        /// </returns>
        internal static string WebJobTypeToString(WebJobType value)
        {
            if (value == WebJobType.Triggered)
            {
                return "triggered";
            }
            if (value == WebJobType.Continuous)
            {
                return "continuous";
            }
            throw new ArgumentOutOfRangeException("value");
        }
    }
}
